import {
  buildNewSvgImages,
  buildOtherSvgImages,
  buildPlayerAvatars,
  drawAllNewCells,
  drawBackgrounds,
  drawBackgroundTopElements,
  drawFlags,
  drawNewAdditionalStuff,
  images,
} from './western-world.utils';
import { shouldRedraw, getMaxSide } from './western-world.utils.helpers';

if (typeof window !== 'undefined') {
  const template = document.createElement('template');
  template.innerHTML = `
    <style>
      :host {
        display: block;
      }
      canvas {
        width: 100%;
        top: 0;
        left: 0;
        transition: filter 0.5s;
      }
    </style>

    <canvas></canvas>
  `;

  class WesternWorld extends HTMLElement {
    constructor() {
      console.log('WesternWorld constructor...');
      super();
      this._shadowRoot = this.attachShadow({ mode: 'open' });
      this._shadowRoot.appendChild(template.content.cloneNode(true));
      this.$canvas = this._shadowRoot.querySelector('canvas');
    }

    static get observedAttributes() {
      return ['language'];
    }

    attributeChangedCallback(name, oldValue, newValue) {
      switch (name) {
        case 'language':
          if (newValue) {
            console.log('language changed...');
            this.fetchImages();
          }
          break;
      }
    }

    allInitialized() {
      return this._showWesternBackground !== undefined && this._game !== undefined;
    }

    set game(value) {
      const areAllInitialized = this.allInitialized();
      const redraw = shouldRedraw(value, this._game);
      this._game = value;
      if (areAllInitialized && redraw) {
        this.drawWesternWorld();
      }
    }

    set showWesternBackground(value) {
      const areAllInitialized = this.allInitialized();
      this._showWesternBackground = value;
      if (areAllInitialized) {
        this.drawWesternWorld();
      }
    }

    fetchImages() {
      console.log('fetch images...');
      const language = this.getAttribute('language');
      images.allImagesProgress.resetLoadingImage();
      console.log(this.allCells);
      Promise.all([
        buildNewSvgImages({
          allCells: this.allCells,
          getMaxSide,
          language,
          PATH_STROKE_WIDTH_FOR_MASK: this.PATH_STROKE_WIDTH,
          PATH_STROKE_WIDTH_FOR_MASK: this.PATH_STROKE_WIDTH_FOR_MASK,
        }),
        buildOtherSvgImages({ language }),
        buildPlayerAvatars(this.playersIconMap),
      ])
        .then(() => {
          console.log('images fetched...');
          this.loaded = true;
          this.dispatchEvent(new CustomEvent('allImagesLoaded', { detail: images }));
          this.drawWesternWorld();
        })
        .catch((e) => console.log('something went wrong', e));
    }

    drawWesternWorld() {
      this.$canvas.width = this.width + this.PADDING_LEFT + this.PADDING_RIGHT;
      this.$canvas.height = this.height + this.PADDING_TOP + this.PADDING_BOTTOM;
      const ctx = this.$canvas.getContext('2d');

      if (this._showWesternBackground) {
        this.$canvas.style.background = 'white';
        drawBackgrounds({
          ctx,
          BACKGROUND_X: this.BACKGROUND_X,
          BACKGROUND_Y: this.BACKGROUND_Y,
          BACKGROUND_WIDTH: this.BACKGROUND_WIDTH,
          BACKGROUND_HEIGHT: this.BACKGROUND_HEIGHT,
        });
      } else {
        this.$canvas.style.background = 'transparent';
      }

      // append all new images;
      drawAllNewCells({
        allCells: this.allCells,
        ctx,
        getMaxSide,
        SIZE_X: this.SIZE_X,
        PADDING_LEFT: this.PADDING_LEFT,
        PADDING_TOP: this.PADDING_TOP,
      });

      if (this._showWesternBackground) {
        drawBackgroundTopElements({
          ctx,
          BACKGROUND_X: this.BACKGROUND_X,
          BACKGROUND_Y: this.BACKGROUND_Y,
          BACKGROUND_WIDTH: this.BACKGROUND_WIDTH,
          BACKGROUND_HEIGHT: this.BACKGROUND_HEIGHT,
        });
      }

      drawNewAdditionalStuff({
        ctx,
        SIZE_X: this.SIZE_X,
        PADDING_LEFT: this.PADDING_LEFT,
        PADDING_TOP: this.PADDING_TOP,
      });

      // draw flags;
      drawFlags({
        allCells: this.allCells,
        ctx,
        game: this._game,
        myIdKey: this.myIdKey,
        SIZE_X: this.SIZE_X,
        PADDING_LEFT: this.PADDING_LEFT,
        PADDING_TOP: this.PADDING_TOP,
      });
    }
  }

  window.customElements.define('western-world', WesternWorld);
}
